'''
Created on 12 Nov 2010

@author: Simon Wilshin
Includes the regular expressions that allow us to look at a variety 
of different marker sets and extract the information we are interested 
in through regular expressions.

Include conventions relating to file headers such as the xml file header.

Includes code to format force plate data in a sensible way facilitating easy access.

Physical Constants are also listed here.

BUG2001 Rear matches +R.... SIMON SIMON
'''

import numpy
from collections import namedtuple
from numpy import pi

import utils


class UnwrittenRoutine(Exception):
    '''
    When I haven't written a section of code an exception should be thrown
    '''
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

# File type conventions
file="file"
directory="directory"

centerOfMass=u"CenterOfMass"
centerOfMassPhaseSignal=u"CenterOfMassPhaseSignal"
centerOfMassPhase=u"CenterOfMassPhase"
trottingStrideSignal=u"TrottingStrideSignal"
vertical=2

genericColourScheme={"phase":"b-","position":"r"}

split={True:"Split",False:"Not Split"}

# Animal Classes
unknown=u"unknown"
biped=u"biped"
quadruped=u"quadruped"
hexaped=u"hexaped"
octoped=u"octoped"

# Stride Classes - Format is dictionary with keys as gaits, then beat pattern expressed as regular expressions
gaitPhases={
            quadruped:{
                  'walk':numpy.array([0,3.0*pi/2.0,pi/2.0,pi]),
                  'trot':numpy.array([0,0,pi,pi]),
                  'pace':numpy.array([0,pi,0,pi])
                  },
            biped:{
                   'pronk':numpy.array([0,0]),
                   'walk':numpy.array([0,pi/2.0])
                   }
           }
gaitTablaux={
             quadruped:utils.getTablaux(4),
             biped:utils.getTablaux(2),
             }

footColours={
             biped:["r","b"],
             quadruped:["r","b","g","c"],
             hexaped:["r","b","g","c","k","m"],
             octoped:["r","b","g","c","k","m","y","w"],
            }

plateColours=["r","b","g","c","k","m","y","w"]

baseForcePlateDirections=["X1","X2","Y1","Y2","Z1","Z2","Z3","Z4"]
colourSchemeBaseForcePlate=['r-','r--','g-','g--','b-','b--','b-.','b:']

spaceDirections3=["X","Y","Z"]
colourSchemeSpace=['r-','g-','b-']
colourSchemeSpaceDashed=['r--','g--','b--']
spaceDirectionIndicies=range(0,3)
spaceDirectionTranslation=dict(zip(spaceDirections3,spaceDirectionIndicies))
pylabColumnMultiplier=100
pylabRowMultiplier=10

pylabImageSizeX=8
pylabImageSizeY=5

ForcePlateIdentifierTuple=namedtuple('ForcePlateIdentifierTuple', 'plate direction', verbose=False)

def makeForcePlateKeys(numberOfForcePlates,forcePlateDirections=baseForcePlateDirections):
    numberOfChannels=len(forcePlateDirections)
    return([ForcePlateIdentifierTuple(
                plate=int(i/numberOfChannels),
                direction=baseForcePlateDirections[i%numberOfChannels]
            )
            for i in range(0,numberOfChannels*numberOfForcePlates)])

def makeForcePlateNumbers(numberOfForcePlates):
    return(range(0,numberOfForcePlates))

def makeResolventForcePlateKeys(numberOfForcePlates):
    return([ForcePlateIdentifierTuple(
             plate=int(i/3),
             direction=spaceDirections3[i%3])
             for i in range(0,3*numberOfForcePlates)])

def makeForcePlateColourScheme(numberOfForcePlates,forcePlateDirections=baseForcePlateDirections,colourScheme=colourSchemeBaseForcePlate):
    return(dict(zip(makeForcePlateKeys(numberOfForcePlates,forcePlateDirections),colourScheme*numberOfForcePlates)))

def makeResolventForcePlateColourScheme(numberOfForcePlates,spaceDirections=spaceDirections3,colourScheme=colourSchemeSpace):
    return(dict(zip(makeResolventForcePlateKeys(numberOfForcePlates),colourScheme*numberOfForcePlates)))

def makeSpaceColourScheme(spaceDirections=spaceDirections3,colourScheme=colourSchemeSpace):
    return(dict(zip(spaceDirections,colourScheme)))

g=9.81
gravity={"X":0.0,"Y":0.0,"Z":-g}
mInmm=0.001

