'''
Created on 20 Nov 2010

@author: Simon Wilshin swilshin@rvc.ac.uk

A class which knows about a working directory and an analysis directory, 
that is a directory where working files are stored (such as raw data) and 
a directory where graphs and other output should be written.
'''

import os
from os.path import join as osjoin

class DirectoryError(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)


class DirectoryAware(object):
    '''
    A class which knows about a working directory and an analysis directory, 
    that is a directory where working files are stored (such as raw data) and 
    a directory where graphs and other output should be written.
    '''


    def __init__(self, baseWorkingDirectory, baseAnalysisDirectory, metaData):
        '''
        Constructor
        '''
        if metaData.has_key("workingDir"):
            self.workingDirectory=osjoin(baseWorkingDirectory,metaData["workingDir"])
        else:
            self.workingDirectory=baseWorkingDirectory
        if metaData.has_key("analysisDir"):
            self.analysisDirectory=osjoin(baseAnalysisDirectory,metaData["analysisDir"])
        else:
            self.analysisDirectory=baseAnalysisDirectory
        # Check the working directory exists and 
        # actually is a directory, if not the user 
        # has probably done something stupid like invoke 
        # the script from the wrong location
        if not os.path.isdir(self.workingDirectory):
            raise DirectoryError("Working directory " + self.workingDirectory + " does not exist, are you running in the correct directory?")
        # Make sure the analysis directory exists and is a 
        # directory, if not throw an exception
        if not os.path.exists(self.analysisDirectory):
            os.makedirs(self.analysisDirectory)
        elif not os.path.isdir(self.analysisDirectory):
            raise DirectoryError("Unable to create analysis directory " + self.analysisDirectory)
            