'''
Created on 15 Mar 2011

@author: Simon Wilshin, swilshin@rvc.ac.uk

Contains text based information, intended to replace a plethora of nearly identical classes.
'''

from os.path import join as osjoin
from yaml import load as yamlload

from mDInterpret import loadMetaData
from baseData import UnsampledBaseData


class TextData(UnsampledBaseData):
    '''
    Contains information about a single nerve block, which legs were blocked 
    and what they were blocked with. Since text data is going to be used 
    everywhere just make it use default number based ids most of the time.
    '''
    
    def __init__(self, workingDirectory, analysisDirectory, metaData, parent=None):
        '''
        Because text data is going to be used extensively and often repeated the id is 
        automatically appended with the key.
        '''
        UnsampledBaseData.__init__(self, workingDirectory, analysisDirectory, metaData, parent)
        loadMetaData("key", self, self.metaData,"key")
        loadMetaData("value", self, self.metaData,"value")
        

    def keys(self):
        return([self.key])

    def has_key(self, k):
        return(k in self.keys())
        
    def __getitem__(self,k):
        if k==self.key:
            return self.value
        else:
            raise(KeyError("No such key in text data, "+str(k)))
        
def getSampleTextData():
    workingDir = osjoin("..","sampleData")    
    analysisDir = osjoin("..","sampleAnalysis","dataContainers")
    metaDataFile = osjoin("..",'sampleYAML','dataContainers','textDataSample.yaml')
    stream = file(metaDataFile, 'r')
    metaData=yamlload(stream)
    stream.close()
    return(TextData(workingDir,analysisDir,metaData))

if __name__=='__main__':
    '''
    Runs a test using the sample data (which is contained wholly in the YAML in this case)
    '''
    print "Processing sample nerve block data"
    sampleTextData=getSampleTextData()
    print "Keys: ", sampleTextData.keys()
    print "Text: ", sampleTextData["animal"]
