'''
Created on 22 May 2012

@author: aspence
'''
from yaml import load as yamlload
from os.path import join as osjoin
from csv import writer
from utils.toroidalGeometry import geodesicDistance7T, geodesicDistance5T
from spiderGaitAblationsSubject import SpiderGaitAblationsSubject
from directoryAware import DirectoryAware
import numpy as np

class SpiderGaitAblationsExperiment(DirectoryAware):
    '''
    The main object for the horse block gait study.
    '''
    def writeTSVData(self):
        # TODO: Add columsn for distanceToWalks<-given dictionnary of gaits, just does it
        # TODO: Add distanceToVault -> this is g5T distance to alt tet with ablated legs left out of the equation
        # TODO: Add distanceToAltTripod -> this is g5T distance to alt tri with ablated legs left out
        #  - Need to lookup legs we are ablating, encode these gaits, figure out g5T, then add.
        labels = ["spider", "trialid", "trialType" ]
        # Cover both trial types:
        rphs=['12RelativeStridePhase','23RelativeStridePhase','34RelativeStridePhase','45RelativeStridePhase','56RelativeStridePhase','67RelativeStridePhase','78RelativeStridePhase','35RelativeStridePhase']
        labels.extend(rphs)
        #labels.extend(self.spiders[0].trials[0].spiderTarsiTrackData.relativeStridePhaseLabels)
        labels.extend(["strideDistanceToAltTet7T","strideDistanceToVault5T","strideDistanceToTripod5T"])
        labels.extend(["speed","startsample","endsample"])
        data = list()
        for sp in self.spiders:
            for t in sp.trials:
                # st loops over stride number within this trial (e.g. 0,1,2,3 ... n)
                for st in range(t.spiderTarsiTrackData["relativeStridePhases"].shape[0]):
                    data.append([sp.name, t.id, t.trialType ]) 
                    if t.spiderTarsiTrackData.trialType=="intact":
                        data[-1].extend(t.spiderTarsiTrackData["relativeStridePhases"][st,:])
                        data[-1].append(np.nan) # Filler for 35 Relphase
                        # Compute distance of this stride to the alternating tetrapod:
                        data[-1].append( geodesicDistance7T(t.spiderTarsiTrackData["relativeStridePhases"][st,:], \
                            self.alttet ) )
                        data[-1].append(np.nan)
                        data[-1].append(np.nan)                            
                    elif t.spiderTarsiTrackData.trialType=="ablated":
                        # Ablated has 1-2 2-3 3-5 5-6 6-7; append 12,23,nan,nan,56,67,nan,35
                        data[-1].extend(t.spiderTarsiTrackData["relativeStridePhases"][st,0:2])
                        data[-1].extend([np.nan,np.nan])
                        data[-1].extend(t.spiderTarsiTrackData["relativeStridePhases"][st,3:5])
                        data[-1].append(np.nan)
                        data[-1].append(t.spiderTarsiTrackData["relativeStridePhases"][st,2])
                        data[-1].append(np.nan) # No dist to alttet in ablated                        
                        # Compute distances to vault and tripd:
                        data[-1].append( geodesicDistance5T(t.spiderTarsiTrackData["relativeStridePhases"][st,:], \
                            self.vault5T ) )
                        data[-1].append( geodesicDistance5T(t.spiderTarsiTrackData["relativeStridePhases"][st,:], \
                            self.tripod5T ) )
                    data[-1].append( t.strideSpeeds[st] )
                    data[-1].append( t.spiderTarsiTrackData.strideIdx[st] )
                    data[-1].append( t.spiderTarsiTrackData.strideIdx[st+1] )
                    
        stepFile=open(osjoin(self.analysisDirectory,'spiderGaitRelativeStridePhases.tsv'), 'wb')
        stepWriter = writer(stepFile, delimiter='\t')
        # Add header here giving order of relative stride phases.
        stepWriter.writerow(labels)
        for d in data:
            stepWriter.writerow(d)
                                                        
    def __init__(self, rootDir, metaDataFileName):
        '''
        The horse nerve block gait experiment is organised into horses and trials, with 
        each trial needing to be loaded seperately and processed.
        '''
        stream = file(osjoin(rootDir,metaDataFileName), 'r')
        metaData=yamlload(stream)
        stream.close()
        #print metaData
        #Experiment.__init__(self, rootDir, rootDir, metaData) # Allow the code to be run with an offset determined by rootDir
        self.alttet = np.array([np.pi,np.pi,np.pi,np.pi,np.pi,np.pi,np.pi])
        self.vault5T = np.array([np.pi,np.pi,0,np.pi,np.pi])
        self.tripod5T = np.array([np.pi,np.pi,np.pi,np.pi,np.pi])
        self.limbs=["1","2","3","4","5","6","7","8"]
        DirectoryAware.__init__(self,rootDir,rootDir,metaData)
        self.name = metaData["experimentName"]
        # Check if we are using per leg phasers trained across the whole experiment:
        #if False:
        if "uselegphasers" in metaData and metaData["uselegphasers"]==True:
            print metaData["uselegphasers"]
            print "Using per leg phasers. Scanning data set for training data"
            # Pseudo code:
            # Scan data set for legs training sections. Create per leg phasers at experiment level. Pass spider tarsi track data objects the training sequences to use to per leg for their phasers. Set flag to use them.
            # Initialize subjects:
            self.spiders=[SpiderGaitAblationsSubject(self.workingDirectory,self.analysisDirectory, spider) for spider in metaData["subjects"]]
            self.legphasertrain = dict()
            for l in self.limbs:
                self.legphasertrain[l] = list()
            for s in self.spiders:
                for t in s.trials:
                    if t.goodTrainingLegs is not None:
                        for l in t.goodTrainingLegs:
                            # Pull the raw data from the spidertarsitrackdata object into the dictionary by limb for whole experiment.
                            # This raw data indexed by limb is what is used to train teh phaser.
                            self.legphasertrain[str(l)].append(t.spiderTarsiTrackData[str(l)])
            for limb, num in zip(self.limbs,[len(l) for l in self.legphasertrain.values()]):
                print "Limb %s has %d training sequences" % (limb, num)
            for s in self.spiders:
                for t in s.trials:
                    t.spiderTarsiTrackData.traindata=self.legphasertrain
                    t.spiderTarsiTrackData.doAnalysis()
                    # Above will have gotten phases a stride cut. So with cutting we can compute stide speeds.
                    t.computeStrideSpeeds()            
        else:
            self.spiders=[SpiderGaitAblationsSubject(self.workingDirectory,self.analysisDirectory, spider) for spider in metaData["subjects"]]
            for s in self.spiders:
                for t in s.trials:
                    t.spiderTarsiTrackData.doAnalysis()
                    # Above will have gotten phases a stride cut. So with cutting we can compute stide speeds.
                    t.computeStrideSpeeds()

def getExampleSpiderGaitAblationsExperiment():
    #return(SpiderGaitAblationsExperiment(osjoin("..","..","..","..","..","data","Spider Learning Data"),'spiderLearning.yaml'))
    return(SpiderGaitAblationsExperiment(osjoin("..","..","..","..","..","data","Spider Learning Data"),'spiderLearning_oneablate.yaml'))
    #return(SpiderGaitAblationsExperiment(osjoin("..","..","..","..","..","data","spiderGaitHovey"),'spiderGaitHovey_abspeed_train.yaml')) # Replace .. with the location of the data to be analysed
    #return(SpiderGaitAblationsExperiment(osjoin("..","..","..","..","..","data","spiderGaitHovey"),'spiderGaitHovey_abspeed_qaqc.yaml')) # Replace .. with the location of the data to be analysed
    #return(SpiderGaitAblationsExperiment("C:\Users\tonia\data\Spider Learning Data"),'spiderLearning_oneablate_onespider.yaml')
if __name__=='__main__':
    '''
    Running the script directly will run our analysis.
    '''
    sga = getExampleSpiderGaitAblationsExperiment()
    sga.writeTSVData()
