from scipy.io import loadmat, savemat
from os import listdir
from numpy import *
from scipy.interpolate import UnivariateSpline
from time import sleep
from scipy.misc import imsave
from PIL import ImageDraw,Image
from sys import argv
from os.path import join as osjoin
from quickhull2d import qhull
import pylab as pl
from os.path import isdir
from os import mkdir

if len(argv)<2:
    wd = "./"
else:
    wd = argv[1]
    print "\n\nOpening Directory: ", argv[1]
allFiles = listdir(wd)
allFiles = sorted([f for f in allFiles if f[:4]=="imgs"])

ang = list()
loc = list()
img = list()
idx = list()
for f in allFiles:
    tmp =  loadmat(osjoin(wd,f))
    for i in range(tmp['data'].shape[1]):
        ang.append(tmp['data'][0][i][0][0][0])
        loc.append(tmp['data'][0][i][1][0])
        img.append(tmp['data'][0][i][2])
        idx.append(tmp['data'][0][i][3][0][0])

ang = array(ang)
loc = array(loc)
idx = array(idx)

trackFiles = [osjoin(wd,f) for f in listdir(wd) if f[-13:]=="_analysis.mat"][0]

dat = loadmat(trackFiles)
px = dat['res'][0][0][1]
py = dat['res'][0][0][2]

pixMax = 0xFFFF

#print img[0].shape

cx = [sum(array(range(i.shape[0]))*sum((pixMax-i),1))/sum((pixMax-i)) for i in img]
cy = [sum(array(range(i.shape[1]))*sum((pixMax-i),0))/sum((pixMax-i)) for i in img]
cy = [52 for i in img]
CoM = vstack([cx,cy]).T

x = array([px,py])
x = x.transpose(1,0,2)
x = x[:,:,:-1]

tx=list()
for ix,theta,x0 in zip(x,ang,loc):
    R = array([[cos(theta),-sin(theta)],[sin(theta),cos(theta)]])
    tx.append((x0+dot(R,ix).T).T)

tx = array(tx)

dx = list()
for ix in tx.transpose(2,1,0):
    dx.append(list())
    for xx in ix:
        dx[-1].append(nan*ones_like(xx))
        idx = array(range(xx.size))
        gdIdx = logical_not(isnan(xx))
        s = UnivariateSpline(idx[gdIdx],xx[gdIdx])
        for i in idx:
            if not isnan(xx[i]):
                dx[-1][-1][i] = s.derivatives(i)[1]

dx = array(dx)

df = sqrt(dx[:,1,:].T**2+dx[:,0,:].T**2)

phi = arctan2(x[:,0,:]-img[0].shape[1]/2.0,x[:,1,:]-img[0].shape[0]/2.0)
idxPhi = array([zip(*sorted(zip(ph,range(phi.shape[1])),key=lambda x: x[0]))[1] for ph in phi])

# Have to sort condition as well
cond0 = less(df,nanmax(df,0)*0.5)
cond = array([zip(*sorted(zip(ph,ic),key=lambda x: x[0]))[1] for ph,ic in zip(phi,cond0)])

polygon = [[sip for sip,sic in zip(ip,ic) if sic] for ip,ic in zip(idxPhi,cond)]
for p in polygon:
    if len(p)>1:
        p.append(p[0])

# Now make polygon into the convex hull
hull = list()
for p,ix in zip(polygon,x):
    hull.append(qhull(ix[:,p].T).T)

def closestOnSegment(x0,x1,r):
    l = dot(x1-x0,(r-x0).T)/float(dot(x1-x0,(x1-x0).T))
    if l<0:
        l=0
    if l>1:
        l=1
    return(l,x0+l*(x1-x0))
        
# Compute stability margins  
margins = list()
for h,c in zip(hull,CoM):
    if h.shape[1]>2:
        margins.append([dot(closestOnSegment(h1,h2,c)[1]-c,closestOnSegment(h1,h2,c)[1].T-c.T) for h1,h2 in zip(h[:,:-1].T,h[:,1:].T)])
    else:
        margins.append(list())
margins = array([sqrt(min(i)) if len(i)>0 else nan for i in margins])

if True:
    for im,h,poly,ix,c,i in zip(img,hull,polygon,x,CoM,range(x.shape[0])):
        pl.figure(1,figsize=(6,10))
        pl.subplot(211)
        pl.imshow(im)
        if len(h)>1:
            for h1,h2 in zip(h[:,:-1].T,h[:,1:].T):
                pl.plot([h1[0],h2[0]],[h1[1],h2[1]],'k',lw=3)
        if not (all(isnan(ix[0])) or all(isnan(ix[1]))):
            pl.scatter(ix[0,:],ix[1,:],c='b',marker='o',s=42)
        if len(poly)>0:
            pl.scatter(ix[0,poly],ix[1,poly],c='r',marker='x',s=111)
        pl.scatter(c[0],c[1],c='k',marker='+',s=64)
        pl.subplot(212)
        pl.vlines(i,nanmin(margins),nanmax(margins),'r',lw=3)
        pl.plot(margins)
        pl.xlim([0,len(margins)-1])
        if not isdir(osjoin(wd,'images')):
            mkdir(osjoin(wd,'images'))
        pl.savefig(osjoin(wd,'images',f+"_analysis_"+str(i).zfill(4)+".png"))
        pl.clf()
        
if True:
    pl.figure()
    ad = osjoin(wd,"stab_analysis")
    if not isdir(ad):
       mkdir(ad)
    pl.plot(margins)
    pl.xlabel("frames (abitrary)")
    pl.ylabel("stability margin (pixels)")
    pl.savefig(osjoin(ad,wd.split("/")[-1]+"_stabilityMargin.png"))
    from pickle import dump
    fle = open(osjoin(ad,wd.split("/")[-1]+"_stabilityMargin.pkl"),'w')
    dump(list(margins),fle)
    savemat(osjoin(ad,wd.split("/")[-1]+"_stabilityMargin.mat"),{'margins':margins} )
    fle.close()
    print osjoin(ad,wd.split("/")[-1]+"_stabilityMargin.mat")
    savemat(osjoin(ad,wd.split("/")[-1]+"_stabilityMargin.mat"),{"margins":margins})


if False:
    print "Writing images..."
    # Make a copy of the images for the video
    imCop = list(img)

    def drawLine(x0,x1,image,v=1):
        c1 = [int(x0[0]),int(x0[1])]
        c2 = [int(x1[0]),int(x1[1])]
        im[c1[0],c1[1]]=v
        im[c2[0],c2[1]]=v
        mvc1=True
        while c1[0]!=c2[0] and c1[1]!=c2[1]:
            if mvc1:
                mvc1=False
                if abs(c1[0]-c2[0])<2.0*abs(c1[1]-c2[1]):
                    c1[1]+=sign(c2[1]-c1[1])
                else:
                    c1[0]+=sign(c2[0]-c1[0])
                im[c1[0],c1[1]]=v
            else:
                mvc1=True
                if abs(c1[0]-c2[0])<2.0*abs(c1[1]-c2[1]):
                    c2[1]+=sign(c1[1]-c2[1])
                else:
                    c2[0]+=sign(c1[0]-c2[0])
                im[c2[0],c2[1]]=v

    # Add lines between feet that are on the ground
    nimCop = list()
    for p,im,ix in zip(polygon,imCop,x):
        if len(p)>1:
            for i,j in zip(p[:-1],p[1:]):
                drawLine([ix[1,i],ix[0,i]],[ix[1,j],ix[0,j]],im,0)
        nimCop.append(array(im))

    imCop = nimCop
            
    # Add dark spots when feet are on the ground
    for i in range(len(imCop)):
        for xx,c in zip(x[i].T,cond0.T):
            if c[i]:
                imCop[i][int(xx[1])-1:int(xx[1])+2,int(xx[0])-1:int(xx[0])+2]=0
                imCop[i][int(xx[1]),int(xx[0])]=pixMax
    i = 0
    
    if not isdir(osjoin(wd,'images')):
        mkdir(osjoin(wd,'images'))
        
    for im in imCop:
        imsave(osjoin(wd,"images/spiderFeet"+str(i).zfill(4)+".png"),im)
        i+=1


