from numpy import *
import sys
import sympy as sp

oldhook = sys.displayhook
sys.displayhook = sp.pprint

def makeSinCos(gaits):
    '''
    Compute the matricies for various gaits
    '''
    Ng = gaits.shape[0]
    C1 = array([[sp.cos(2*sp.pi*subgait) for subgait in gait] for gait in gaits])
    S1 = array([[sp.sin(2*sp.pi*subgait) for subgait in gait] for gait in gaits])
    return(C1,S1)

def makeEquations(C1,S1,gamma,epsilon,sigma,omega,eta,rhs):
    return(sp.Matrix(
           sum(gamma*C1,1) +
           sum(epsilon*S1,1) +
           diagonal(dot(S1,dot(sigma,S1.transpose()))) +
           diagonal(dot(C1,dot(omega,C1.transpose()))) +
           diagonal(dot(S1,dot(eta,C1.transpose()))) +
           - array(rhs)
    ))
    
def makeDerivative(C1,S1,gamma,epsilon,sigma,omega,eta):
    return(sp.Matrix(
        -gamma*S1 +
        epsilon*C1 +
        2*dot(S1,sigma)*C1 +
        -2*dot(C1,omega)*S1 +
        dot(C1,eta.transpose())*C1 - dot(S1,eta)*S1
    ))

def makeDDerivative(C1,S1,gamma,epsilon,sigma,omega,eta):
    return(sp.Matrix(
        -gamma*C1 +
        -epsilon*S1 +
        -2*dot(S1,sigma)*S1 +
        -2*dot(C1,omega)*C1 +
        -dot(C1,eta.transpose())*S1 - dot(S1,eta)*C1
    ))

def makeODDDerivative(C1,S1,gamma,epsilon,sigma,omega,eta):
    beta = [2,0,1]
    alpha = [1,2,0]
    return(sp.Matrix(
                      (2*C1[:,beta]*sigma[beta,alpha]-S1[:,beta]*eta.transpose()[beta,alpha])*C1[:,alpha] +
                      (2*S1[:,beta]*omega[beta,alpha]-C1[:,beta]*eta[beta,alpha])*S1[:,alpha]
                     ))

g3T = 1.0/4.0*array([
                     [3,2,1],
                     [2,4,2],
                     [1,2,3]
                    ])


# Gaits and Costs
walkGaits = sp.Rational(1,4)*array([
                                    [2,3,2],
                                    [2,1,2],
                                    [1,1,1],
                                    [3,3,3],
                                    [1,2,3],
                                    [3,2,1]
                                   ])
                                   
walkLongScores = array([
                        0,
                        0,
                        1,
                        1,
                        1,
                        -1
                       ])

pronkGaits = array([
                    [0,0,0]
                   ])

otherStationaryGait = sp.Rational(1,6)*array([
                                              [sp.Rational(3,2),1,5]
                                             ])
                                      
# Parameters
gamma = array([sp.Symbol('g0'),sp.Symbol('g1'),sp.Symbol('g2')])
epsilon = array([sp.Symbol('e0'),sp.Symbol('e1'),sp.Symbol('e2')])
sigma = array([
               [0,sp.Symbol('s10'),sp.Symbol('s20')],
               [sp.Symbol('s10'),0,sp.Symbol('s21')],
               [sp.Symbol('s20'),sp.Symbol('s21'),0]
              ]) 

omega = array([
               [0,sp.Symbol('w10'),sp.Symbol('w20')],
               [sp.Symbol('w10'),0,sp.Symbol('w21')],
               [sp.Symbol('w20'),sp.Symbol('w21'),0]
              ])

eta = array([
             [0,sp.Symbol('eta01'),sp.Symbol('eta02')],
             [sp.Symbol('eta10'),0,sp.Symbol('eta12')],
             [sp.Symbol('eta20'),sp.Symbol('eta21'),0]
            ])

C1,S1 = makeSinCos(walkGaits)

e = makeEquations(C1,S1,gamma,epsilon,sigma,omega,eta,walkLongScores)
de = makeDerivative(C1,S1,gamma,epsilon,sigma,omega,eta)
dde = makeDDerivative(C1,S1,gamma,epsilon,sigma,omega,eta)
ddde = makeODDDerivative(C1,S1,gamma,epsilon,sigma,omega,eta)

C1,S1 = makeSinCos(pronkGaits)

pe = makeEquations(C1,S1,gamma,epsilon,sigma,omega,eta,[0]) 
pde = makeDerivative(C1,S1,gamma,epsilon,sigma,omega,eta)
pdde = makeDDerivative(C1,S1,gamma,epsilon,sigma,omega,eta)

C1,S1 = makeSinCos(otherStationaryGait)

osge = makeEquations(C1,S1,gamma,epsilon,sigma,omega,eta,[0]) 
osgde = makeDerivative(C1,S1,gamma,epsilon,sigma,omega,eta,)
osgdde = makeDDerivative(C1,S1,gamma,epsilon,sigma,omega,eta) 

def applyTrans(trans,e):
    for s,t in trans:
        e = e.subs(s,t)
    return(e)

def prntLastTrans(trans):
    print str(trans[-1][0]) + " = " + str(trans[-1][1])

# Save the necessary transformations
trans = []

# Solve Stability Equation
trans.append((epsilon[1],epsilon[1]-sp.Rational(1,2)*(e[2]-e[3])))
prntLastTrans(trans)
e = applyTrans(trans,e)

trans.append((sigma[2,1],sigma[2,1]-sp.Rational(1,2)*e[2]))
prntLastTrans(trans)
e = applyTrans(trans,e)

trans.append((omega[2,0],omega[2,0]-sp.Rational(1,4)*(e[0]+e[1])))
prntLastTrans(trans)
e = applyTrans(trans,e)

trans.append((eta[1,2],eta[1,2]-e[0]))
prntLastTrans(trans)
e = applyTrans(trans,e)

trans.append((eta[1,2],eta[1,2]-e[0]))
prntLastTrans(trans)
e = applyTrans(trans,e)

trans.append((sigma[2,0],sigma[2,0]+sp.Rational(1,4)*(e[4]+e[5])))
prntLastTrans(trans)
e = applyTrans(trans,e)

trans.append((eta[2,1],eta[2,1]-e[4]))
prntLastTrans(trans)
e = applyTrans(trans,e)

# Check
print e # Should be zero

de = applyTrans(trans,de)

# Now make the walk gait a stationary
trans.append((eta[2,0],eta[2,0]-de[5,0]))
prntLastTrans(trans)
de = applyTrans(trans,de)

trans.append((eta[0,2],eta[0,2]-de[5,2]))
prntLastTrans(trans)
de = applyTrans(trans,de)

trans.append((sigma[1,0],sigma[1,0]-sp.Rational(1,4)*de[5,1]))
prntLastTrans(trans)
de = applyTrans(trans,de)

dde = applyTrans(trans,dde)

print "dde before parametrization:", dde
# Now parameterise this so it will be a minima
z = sp.Symbol('z')
y = sp.Symbol('y')

trans.append((eta[0,1],eta[0,1]+dde[5,0]-y))
prntLastTrans(trans)
dde = applyTrans(trans,dde)

trans.append((gamma[1],gamma[1]+sp.Rational(1,2)*(dde[5,2]-y)))
prntLastTrans(trans)
dde = applyTrans(trans,dde)

trans.append((epsilon[0],epsilon[0]+dde[5,1]-z))
prntLastTrans(trans)
dde = applyTrans(trans,dde)

print "dde after parametrisation", dde

e = applyTrans(trans,e)
de = applyTrans(trans,de)

pe =  applyTrans(trans,pe)
pde =  applyTrans(trans,pde)

# Make the all maximally unstable walks stationary
trans.append((omega[2,1],omega[2,1]+sp.Rational(1,4)*de[4,2]))
prntLastTrans(trans)
de = applyTrans(trans,de)

trans.append((epsilon[2],epsilon[2]-sp.Rational(1,4)*de[4,1]))
prntLastTrans(trans)
de = applyTrans(trans,de)

trans.append((omega[1,0],omega[1,0]-sp.Rational(1,4)*de[4,0]))
prntLastTrans(trans)
de = applyTrans(trans,de)

trans.append((y,y+de[3,1]))
prntLastTrans(trans)
de = applyTrans(trans,de)

trans.append((gamma[2],gamma[2]-de[3,2]))
prntLastTrans(trans)
de = applyTrans(trans,de)

trans.append((eta[1,0],eta[1,0]+de[3,0]))
prntLastTrans(trans)
de = applyTrans(trans,de)

trans.append((gamma[0],gamma[0]+sp.Rational(1,2)*de[2,0]))
prntLastTrans(trans)
de = applyTrans(trans,de)

# Apply transformations
pe =  applyTrans(trans,pe)
pde =  applyTrans(trans,pde)
de =  applyTrans(trans,de)
dde =  applyTrans(trans,dde)
osge = applyTrans(trans,osge)
osgde = applyTrans(trans,osgde)
osgdde = applyTrans(trans,osgdde)
ddde = applyTrans(trans,ddde)

Cs = array([[sp.Symbol('cos(x[0])'),sp.Symbol('cos(x[1])'),sp.Symbol('cos(x[2])')]])
Ss = array([[sp.Symbol('sin(x[0])'),sp.Symbol('sin(x[1])'),sp.Symbol('sin(x[2])')]])

# Calculate Potential
V = makeEquations(Cs,Ss,gamma,epsilon,sigma,omega,eta,[0])[0]
dV = makeDerivative(Cs,Ss,gamma,epsilon,sigma,omega,eta)
ddV = makeDDerivative(Cs,Ss,gamma,epsilon,sigma,omega,eta)

V =  applyTrans(trans,V)
dV =  applyTrans(trans,dV)

sys.displayhook = oldhook
V = sp.simplify(V)
dV.simplify()
ddV.simplify
print "\nV"
print V
print "\ndV"
print dV
print "\nddV"
print ddV

def Potential(x,z):
    return(
        -cos(x[1])*sin(x[0])*z/2 +
        cos(x[1])*sin(x[2])*z/2 +
        sin(x[0])*sin(x[1])/2 -
        sin(x[0])*z/2 +
        sin(x[0])/2 +
        sin(x[1])*sin(x[2])/2 +
        sin(x[2])*z/2 -
        sin(x[2])/2
       )

def dPotential(x,z):
    return(array(
            [
             cos(x[0])*(
                        -cos(x[1])*z +
                        sin(x[1]) -
                        z +
                        1
                       )/2,
             cos(x[1])*sin(x[0])/2 +
             cos(x[1])*sin(x[2])/2 +
             sin(x[0])*sin(x[1])*z/2 -
             sin(x[1])*sin(x[2])*z/2,
             cos(x[2])*(
                        cos(x[1])*z +
                        sin(x[1]) +
                        z -
                        1
                       )/2
            ]))

walkGaits = 1.0/4.0*array([
                                    [2,3,2],
                                    [2,1,2],
                                    [1,1,1],
                                    [3,3,3],
                                    [1,2,3],
                                    [3,2,1]
                                   ])    
    
# Check scores at z=0
print [Potential(2.0*pi*w,0) for w in walkGaits]
print abs(array([Potential(2.0*pi*w,0) for w in walkGaits]) - walkLongScores) < 1e-10

# Check scores at z=0.5
print [Potential(2.0*pi*w,0.5) for w in walkGaits]
print abs(array([Potential(2.0*pi*w,0.5) for w in walkGaits]) - walkLongScores) < 1e-10

# Check scores
print [Potential(2.0*pi*w,1.0) for w in walkGaits]
print abs(array([Potential(2.0*pi*w,1.0) for w in walkGaits]) - walkLongScores) < 1e-10

# Check Gradients
print [dPotential(2.0*pi*w,0) for w in walkGaits]
print abs(array([dPotential(2.0*pi*w,0) for w in walkGaits])) < 1e-10

# Check Gradients
print [dPotential(2.0*pi*w,1.0) for w in walkGaits]
print abs(array([dPotential(2.0*pi*w,1.0) for w in walkGaits])) < 1e-10     

#Check Gradients match up
print array([dPotential(2.0*pi*w+[0.001,0.0,0.0],1.0)-dPotential(2.0*pi*w,1.0) for w in walkGaits])[5,0]/0.001
print array([dPotential(2.0*pi*w+[0.0,0.001,0.0],1.0)-dPotential(2.0*pi*w,1.0) for w in walkGaits])[5,1]/0.001
print array([dPotential(2.0*pi*w+[0.0,0.0,0.001],1.0)-dPotential(2.0*pi*w,1.0) for w in walkGaits])[5,2]/0.001

# Now show parameters
p = []
p.extend(sp.Matrix(gamma).mat)
p.extend(sp.Matrix(epsilon).mat)
p.extend(sp.Matrix(sigma).mat)
p.extend(sp.Matrix(omega).mat)
p.extend(sp.Matrix(eta).mat)
p = sp.Matrix(p)
op = p
p = applyTrans(trans,p)
for pi,qi in zip(p,op):
    print qi, " = ", pi
