'''
Created on 4 Feb 2011

@author: swilshin
'''

import numpy as np       
from scipy.weave import inline
from scipy.weave import converters
#from weave import inline
#from weave import converters

def randomIndexer1D(a,idx):
    '''
    Enables fast random indexing of numpy arrays.
    '''
    indexerCode='''
    int i = 0;
    for(i = 0; i < n; ++i)
    {
        ret(i) = t_a(t_idx(i));
    }
    '''
    t_idx = np.array(idx)
    n = t_idx.size #@UnusedVariable Used inline in weave
    ret = np.zeros(t_idx.size)
    t_a = np.array(a) #@UnusedVariable Used inline in weave
    inline(indexerCode,['t_a','n','t_idx','ret'],type_converters=converters.blitz)
    return(ret)

def randomIndexer2D(a,idx):
    '''
    Enables fast random indexing of numpy arrays.
    '''
    indexerCode='''
    int i = 0;
    int j = 0;
    for(i = 0; i < n; ++i)
    {
        for(j = 0; j < m; ++j)
        {
            ret(m*i+j) = t_a(M*i+t_idx(j));
        }
    }
    '''
    t_a = np.array(a)
    n = t_a.shape[0]
    t_idx = np.array(idx)
    m = t_idx.size
    M = t_a.shape[1] #@UnusedVariable Used inline in weave
    t_a = np.array(t_a.reshape(t_a.size)) #@UnusedVariable Used inline in weave
    ret = np.zeros(n*m)
    inline(indexerCode,['t_a','n','m','M','t_idx','ret'],type_converters=converters.blitz)
    ret = ret.reshape((n,m))
    return(ret)

# Testing
def testRandomIndexer1D():
    print "Testing randomIndexer1D"
    ri = randomIndexer1D([5,6,9],[0,2])
    t1 = ri[0]==5
    print "Test 0001 Passed: ", t1
    t2 = ri[1]==9
    print "Test 0002 Passed: ", t2
    testsPassed = t1 and t2
    return(testsPassed)

def testRandomIndexer2D():
    print "Testing randomIndexer2D"
    ri = randomIndexer2D([[5,6,9],[8,3,0]],[0,2])
    t1 = ri[0][0]==5
    print "Test 0001 Passed: ", t1
    t2 = ri[0][1]==9
    print "Test 0002 Passed: ", t2
    t3 = ri[1][0]==8
    print "Test 0003 Passed: ", t3
    t4 = ri[1][1]==0
    print "Test 0004 Passed: ", t4
    testsPassed = t1 and t2 and t3 and t4
    return(testsPassed)

def testRandomIndexer():
    print "Testing random indexer module, True indicates test passed."        
    t1=testRandomIndexer1D()
    t2=testRandomIndexer2D()
    testsPassed=t1 and t2
    if testsPassed:
        print "All tests PASSED in module randomIndexer."
    else:
        print "Some tests FAILED in module randomIndexer!"
    return(testsPassed)

if __name__=='__main__':
    '''
    Run some tests, should print True if everything worked
    '''
    testRandomIndexer()